<?php
/**
 * DBShop 电子商务系统
 *
 * ==========================================================================
 * @link      https://www.dbshop.net/
 * @copyright 北京珑大钜商科技有限公司，并保留所有权利。
 * @license   https://www.dbshop.net/license.html License
 * ==========================================================================
 *
 * @author    静静的风 <baron@loongdom.cn>
 *
 */

namespace Admin\Controller;

use Admin\Entity\SelfLocation;
use Admin\Entity\SelfLocationMember;
use Admin\Form\SelfLocationForm;
use Admin\Form\SelfLocationMemberChangePasswordForm;
use Admin\Form\SelfLocationMemberForm;
use Admin\Service\SelfLocationManager;
use Admin\Service\SelfLocationMemberManager;
use Doctrine\ORM\EntityManager;
use Laminas\Mvc\I18n\Translator;
use Laminas\View\Model\JsonModel;
use PluginLocationStore\Entity\PluginLocationStoreOrder;

/**
 * 自提点
 * Class SelfLocationController
 * @package Admin\Controller
 */
class SelfLocationController extends AdminActionController
{
    private $translator;
    private $entityManager;
    private $selfLocationManager;
    private $selfLocationMemberManager;

    public function __construct(
        Translator      $translator,
        EntityManager   $entityManager,
        SelfLocationManager $selfLocationManager,
        SelfLocationMemberManager $selfLocationMemberManager
    )
    {
        $this->translator   = $translator;
        $this->entityManager= $entityManager;
        $this->selfLocationManager = $selfLocationManager;
        $this->selfLocationMemberManager = $selfLocationMemberManager;
    }

    public function indexAction()
    {
        $selfLocationList = $this->entityManager->getRepository(SelfLocation::class)->findBy([], ['locationSort' => 'ASC', 'locationId' => 'DESC']);

        return ['locationList' => $selfLocationList];
    }

    /**
     * 自提点添加
     * @return array|\Laminas\Http\Response
     * @throws \Doctrine\ORM\ORMException
     * @throws \Doctrine\ORM\OptimisticLockException
     */
    public function addAction()
    {
        $form = new SelfLocationForm();

        if($this->getRequest()->isPost()) {
            $data = $this->params()->fromPost();
            $form->setData($data);
            if ($form->isValid()) {
                $data = $form->getData();
                $this->selfLocationManager->addSelfLocation($data);

                $this->adminCommon()->addOperLog(sprintf($this->translator->translate('自提点 %s 添加成功!'), $data['locationName']), $this->translator->translate('自提点设置'));

                return $this->redirect()->toRoute('self-location');
            }
        }

        return ['form' => $form, 'region' => $this->adminCommon()->getRegionSub()];
    }

    /**
     * 自提点编辑
     * @return array|\Laminas\Http\Response
     * @throws \Doctrine\ORM\ORMException
     * @throws \Doctrine\ORM\OptimisticLockException
     */
    public function editAction()
    {
        $locationId     = (int) $this->params()->fromRoute('id', 0);
        $locationInfo   = $this->entityManager->getRepository(SelfLocation::class)->findOneBy(['locationId' => $locationId]);
        if($locationInfo == null) {
            $this->flashMessenger()->addWarningMessage($this->translator->translate('该自提点不存在!'));
            return $this->redirect()->toRoute('self-location');
        }

        $form = new SelfLocationForm();

        if($this->getRequest()->isPost()) {
            $data = $this->params()->fromPost();
            $form->setData($data);
            if ($form->isValid()) {
                $data = $form->getData();
                $this->selfLocationManager->editSelfLocation($data, $locationInfo);

                $this->adminCommon()->addOperLog(sprintf($this->translator->translate('自提点 %s 编辑成功!'), $data['locationName']), $this->translator->translate('自提点设置'));

                return $this->redirect()->toRoute('self-location');
            }
        } else $form->setData($locationInfo->valuesArray());

        return ['locationInfo' => $locationInfo, 'form' => $form, 'region' => $this->adminCommon()->getRegionSub()];
    }

    /**
     * 删除自提点
     * @return \Laminas\Http\Response
     * @throws \Doctrine\ORM\ORMException
     * @throws \Doctrine\ORM\OptimisticLockException
     */
    public function deleteAction()
    {
        if(!$this->adminCommon()->validatorCsrf()) return $this->adminCommon()->toReferer();

        $locationId = (int) $this->params()->fromRoute('id', 0);
        $locationInfo   = $this->entityManager->getRepository(SelfLocation::class)->findOneBy(['locationId' => $locationId]);
        if($locationInfo == null) {
            $this->flashMessenger()->addWarningMessage($this->translator->translate('该自提点不存在!'));
            return $this->redirect()->toRoute('self-location');
        }

        $locationMember = $this->entityManager->getRepository(SelfLocationMember::class)->findOneBy(['locationId' => $locationId]);
        if ($locationMember) {
            $this->flashMessenger()->addWarningMessage($this->translator->translate('该自提点下还有核销人员，不能删除!'));
            return $this->redirect()->toRoute('self-location');
        }

        $this->selfLocationManager->deleteSelfLocation($locationInfo);

        $this->adminCommon()->addOperLog(sprintf($this->translator->translate('自提点 %s 删除成功!'), $locationInfo->getLocationName()), $this->translator->translate('自提点设置'));

        return $this->adminCommon()->toReferer();
    }

    /**
     * 获取地理编码（暂未使用）
     * @return JsonModel
     */
    public function ajaxLocationGeoCodingAction()
    {
        $locationAddress = urlencode($this->params()->fromQuery('locationAddress'));
        $locationData = file_get_contents('https://api.map.baidu.com/geocoding/v3/?address='.$locationAddress.'&output=json&ak=&ret_coordtype=gcj02ll');
        return $this->getResponse()->setContent($locationData);
    }

    /**
     * 核销人员列表
     * @return array
     */
    public function memberListAction()
    {
        $memberList     = $this->entityManager->getRepository(SelfLocationMember::class)->findMemberList([]);
        //检查是否启用了自提点插件
        $locationStore  = class_exists(PluginLocationStoreOrder::class);

        return ['memberList' => $memberList, 'locationStore' => $locationStore];
    }

    /**
     * 核销人员添加
     * @return SelfLocationMemberForm[]|\Laminas\Http\Response
     * @throws \Doctrine\ORM\ORMException
     * @throws \Doctrine\ORM\OptimisticLockException
     */
    public function addMemberAction()
    {
        $form = new SelfLocationMemberForm($this->entityManager, null);
        $form->get('locationId')->setValueOptions($this->adminCommon()->selfLocationOptions());

        if($this->getRequest()->isPost()) {
            $data = $this->params()->fromPost();
            $form->setData($data);
            if ($form->isValid()) {
                $data = $form->getData();
                $this->selfLocationMemberManager->addSelfLocationMember($data);

                $this->adminCommon()->addOperLog(sprintf($this->translator->translate('核销员 %s 添加成功!'), $data['memberName']), $this->translator->translate('自提点核销员'));

                return $this->redirect()->toRoute('self-location', ['action' => 'memberList']);
            }
        }

        return ['form' => $form];
    }

    /**
     * 编辑核销人员
     * @return array|\Laminas\Http\Response
     * @throws \Doctrine\ORM\ORMException
     * @throws \Doctrine\ORM\OptimisticLockException
     */
    public function editMemberAction()
    {
        $memberId = (int) $this->params()->fromRoute('id', -1);
        $memberInfo = $this->entityManager->getRepository(SelfLocationMember::class)->findOneBy(['memberId' => $memberId]);
        if ($memberInfo == null) {
            $this->flashMessenger()->addWarningMessage($this->translator->translate('核销人员不存在!'));
            return $this->redirect()->toRoute('self-location', ['action' => 'memberList']);
        }

        $form = new SelfLocationMemberForm($this->entityManager, $memberInfo, 'edit');
        $form->get('locationId')->setValueOptions($this->adminCommon()->selfLocationOptions());

        if($this->getRequest()->isPost()) {
            $data = $this->params()->fromPost();
            $form->setData($data);
            if ($form->isValid()) {
                $data = $form->getData();
                $this->selfLocationMemberManager->editSelfLocationMember($data, $memberInfo);

                $this->adminCommon()->addOperLog(sprintf($this->translator->translate('核销员 %s 编辑成功!'), $data['memberName']), $this->translator->translate('自提点核销员'));

                return $this->redirect()->toRoute('self-location', ['action' => 'memberList']);
            }
        } else $form->setData($memberInfo->valuesArray());

        return ['form' => $form, 'member' => $memberInfo];
    }

    /**
     * 重置密码
     * @return array|\Laminas\Http\Response
     * @throws \Doctrine\ORM\ORMException
     * @throws \Doctrine\ORM\OptimisticLockException
     */
    public function changeMemberPasswordAction()
    {
        $memberId = (int) $this->params()->fromRoute('id', -1);
        $memberInfo = $this->entityManager->getRepository(SelfLocationMember::class)->findOneBy(['memberId' => $memberId]);
        if ($memberInfo == null) {
            $this->flashMessenger()->addWarningMessage($this->translator->translate('核销人员不存在!'));
            return $this->redirect()->toRoute('self-location', ['action' => 'memberList']);
        }

        $form = new SelfLocationMemberChangePasswordForm();

        if ($this->getRequest()->isPost()) {
            $data = $this->params()->fromPost();
            $form->setData($data);
            if ($form->isValid()) {
                $data = $form->getData();
                $this->selfLocationMemberManager->changeMemberPassword($data['memberPassword'], $memberInfo);

                $this->adminCommon()->addOperLog(sprintf($this->translator->translate('核销员 %s 密码修改成功!'), $memberInfo->getMemberName()), $this->translator->translate('自提点核销员'));

                return $this->redirect()->toRoute('self-location', ['action' => 'memberList']);
            }
        }

        return ['form' => $form, 'member' => $memberInfo];
    }

    /**
     * 核销人员删除
     * @return \Laminas\Http\Response
     * @throws \Doctrine\ORM\ORMException
     * @throws \Doctrine\ORM\OptimisticLockException
     */
    public function deleteMemberAction()
    {
        if(!$this->adminCommon()->validatorCsrf()) return $this->redirect()->toRoute('self-location', ['action' => 'memberList']);

        $memberId = (int) $this->params()->fromRoute('id', -1);
        $memberInfo = $this->entityManager->getRepository(SelfLocationMember::class)->findOneBy(['memberId' => $memberId]);
        if ($memberInfo == null) {
            $this->flashMessenger()->addWarningMessage($this->translator->translate('核销人员不存在!'));
            return $this->redirect()->toRoute('self-location', ['action' => 'memberList']);
        }

        $this->selfLocationMemberManager->deleteSelfLocationMember($memberInfo);

        $this->adminCommon()->addOperLog(sprintf($this->translator->translate('核销员 %s 删除成功!'), $memberInfo->getMemberName()), $this->translator->translate('自提点核销员'));

        return $this->adminCommon()->toReferer();
    }
}